from __future__ import absolute_import
import json
from celery.result import AsyncResult

from django.views.generic.base import View
from django.http import HttpResponse, Http404
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt

from .models import Service, Task
from .tasks import deploy_task
from .utils.core import absolute_url
from .utils.repo import repository_parser


class DeployView(View):

    """Main view receive POST Hook from repository"""

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        try:
            self.service = Service.objects.get(secret_key=kwargs['secret_key'])
        except Exception:
            raise Http404

        self.service_pk = str(self.service.pk)
        return super(DeployView, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        json_string = request.POST['payload'].decode(
            'string_escape').replace('\n', '')
        data = json.loads(json_string)

        if(repository_parser(data, self.service).check_branch()):
            host = absolute_url(request).build()
            job = deploy_task.apply_async((host, json_string, self.service),
                                          countdown=1)
            Task.objects.create(name=job.id, service=self.service)

        return HttpResponse(status=200)


class DeployStatusView(DeployView):

    def post(self, request, *args, **kwargs):
        data = dict()
        if self.service.has_queue():
            task_id = self.service.task_set.all()[0]
            task = AsyncResult(task_id.name)
            data = task.result or dict(status=task.state)

        data['queue'] = self.service.task_set.all().count()
        json_data = json.dumps(data)
        return HttpResponse(json_data, mimetype='application/json')




from .ftp import ftp_check
from .repo import bitbucket_check, github_check
from ftp_deploy.conf import *


class service_check(object):

    """Service check class, what group together all checking points.
       Return 'fails' and 'message' lists"""

    def __init__(self, service):
        self.service = service
        self.message = list()
        self.fails = [False, False, False, False]

    def check_log(self):
        """Check logs"""
        if self.service.pk is not None:
            log_fail = self.service.log_set.all().filter(
                status=False).filter(skip=False).count()
            if log_fail:
                self.message.append('<b>Log</b>: Deploy Fails(%d)' % log_fail)
                self.fails[0] = True

    def check_repo(self):
        """Check repositories connection, along with POST Hook"""
        if self.service.repo_source == 'bb':
            repo = bitbucket_check(
                BITBUCKET_SETTINGS['username'], BITBUCKET_SETTINGS['password'],
                self.service)
        elif self.service.repo_source == 'gh':
            repo = github_check(
                GITHUB_SETTINGS['username'], GITHUB_SETTINGS['password'],
                self.service)

        repo_fail, repo_fail_message = repo.check_all()
        if repo_fail:
            self.message.append(repo_fail_message)
            self.fails[1] = True

        hook_fail, hook_fail_message = repo.check_hook_exist()
        if hook_fail:
            self.message.append(hook_fail_message)
            self.fails[2] = True

    def check_ftp(self):
        """Check FTP connection"""
        ftp = ftp_check(self.service.ftp_host, self.service.ftp_username,
                        self.service.ftp_password, self.service.ftp_path)
        ftp_fail, ftp_fail_message = ftp.check_all()
        ftp.quit()

        if ftp_fail:
            self.message.append(ftp_fail_message)
            self.fails[3] = True

    def check_all(self):
        self.check_log()
        self.check_repo()
        self.check_ftp()

        return self.fails, self.message


class absolute_url(object):

    """Build absolute url to root url without trailing slash"""

    def __init__(self, request):
        self.request = request

    def build(self):
        return self.request.build_absolute_uri('/')[:-1]


class LockError(Exception):

    """Exception if service is locked"""

    def __str__(self):
        return 'Deploy failed because service is Locked!'



from django.views.generic import ListView, UpdateView, DeleteView, CreateView
from django.core.urlresolvers import reverse_lazy
from django.contrib import messages

from braces.views import LoginRequiredMixin

from ftp_deploy.models import Notification
from ftp_deploy.server.forms import NotificationForm


class NotificationView(LoginRequiredMixin, ListView):

    model = Notification
    template_name = 'ftp_deploy/notification/notification.html'
    context_object_name = 'notifications'


class NotificationAddView(LoginRequiredMixin, CreateView):

    """View for add notifications"""

    model = Notification
    form_class = NotificationForm
    success_url = reverse_lazy('ftpdeploy_notification')
    template_name = "ftp_deploy/notification/form.html"

    def form_valid(self, form):
        messages.add_message(self.request, messages.SUCCESS,
                             'Notification has been added.')
        return super(NotificationAddView, self).form_valid(form)


class NotificationEditView(LoginRequiredMixin, UpdateView):

    """View for edit notifications"""

    model = Notification
    form_class = NotificationForm
    success_url = reverse_lazy('ftpdeploy_notification')
    template_name = "ftp_deploy/notification/form.html"

    def get_context_data(self, **kwargs):
        context = super(NotificationEditView, self).get_context_data(**kwargs)
        emails = self.get_object().get_email_list()
        context['emails'] = emails
        return context

    def form_valid(self, form):
        messages.add_message(self.request, messages.SUCCESS,
                             'Notification has been updated.')
        return super(NotificationEditView, self).form_valid(form)


class NotificationDeleteView(LoginRequiredMixin, DeleteView):

    """View for delete services"""

    model = Notification
    success_url = reverse_lazy('ftpdeploy_notification')
    template_name = "ftp_deploy/notification/delete.html"

    def delete(self, request, *args, **kwargs):
        messages.add_message(request, messages.SUCCESS,
                             'Notification has been removed.')
        return super(NotificationDeleteView, self).delete(request, *args,
                                                          **kwargs)



import os
from ftplib import FTP

from .decorators import check


class ftp_connection(object):
    """FTP Connection helper. Provide methods to manage ftp resources"""
    def __init__(self, host, username, password, ftp_path):
        self.host = host
        self.username = username
        self.password = password
        self.ftp_path = self.encode(ftp_path)
        self.connected = 0

    def connect(self):
        """Initialize FTP Connection"""
        self.ftp = FTP(self.host)
        self.ftp.login(self.username, self.password)
        self.connected = 1

    def create_file(self, file_path, content):
        """Create file populated with 'content'
            and save to 'file_path' location"""
        file_path = self.encode(file_path)
        self.ftp.storbinary('STOR ' + self.ftp_path + file_path, content)

    def remove_file(self, file_path):
        """Remove file from 'file_path' location,
            and clear empty directories"""
        file_path = self.encode(file_path)
        self.ftp.delete(self.ftp_path + file_path)
        dirname = file_path.split('/')
        for i in xrange(len(dirname)):
            current = '/'.join(dirname[:-1 - i])
            try:
                self.ftp.rmd(self.ftp_path + current)
            except Exception, e:
                return False

    def make_dirs(self, file_path):
        """ Create FTP tree directories based on 'file_path'"""
        file_path = self.encode(file_path)
        dirname = os.path.dirname(file_path).split('/')
        for i in xrange(len(dirname)):
            current = '/'.join(dirname[:i + 1])
            try:
                self.ftp.dir(self.ftp_path + current)
            except Exception, e:
                self.ftp.mkd(self.ftp_path + current)

    def encode(self, content):
        """Encode path string"""
        return str(content).encode('ascii', 'ignore')

    def quit(self):
        """Close FTP Connection"""
        return self.ftp.quit() if self.connected else False


class ftp_check(ftp_connection):

    """Check FTP Connection, return True if fail"""

    @check('FTP')
    def check_ftp_login(self):
        self.connect()

    @check('FTP')
    def check_ftp_path(self):
        self.ftp.cwd(self.ftp_path)
        self.ftp.cwd('/')

    def check_all(self):

        status = self.check_ftp_login()
        if status[0] is True:
            return status

        status = self.check_ftp_path()
        if status[0] is True:
            return status

        return False, ''



from django.conf.urls import patterns, url
from .views import DeployView, DeployStatusView

urlpatterns = patterns(
    '',
    url(r'^deploy/status/(?P<secret_key>.+)$', DeployStatusView.as_view(),
        name='ftpdeploy_deploy_status'),
    url(r'^deploy/(?P<secret_key>.+)$', DeployView.as_view(),
        name='ftpdeploy_deploy'),
)



from django.db import models


class Notification(models.Model):
    name = models.CharField(max_length=200, unique=True)
    success = models.TextField(blank=True)
    fail = models.TextField(blank=True)
    commit_user = models.CharField('Commit User', max_length=50, blank=True)
    deploy_user = models.CharField('Deploy User', max_length=50, blank=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.name

    def get_email_list(self):
        emails = dict()

        for email in self.get_success():
            emails[email] = dict()
            emails[email]['success'] = True

        for email in self.get_fail():
            try:
                emails[email]['fail'] = True
            except KeyError, e:
                emails[email] = dict()
                emails[email]['fail'] = True

        return emails

    def get_success(self):
        return filter(None, self.success.split(','))

    def get_fail(self):
        return filter(None, self.fail.split(','))

    def commit_user_success(self):
        return True if '0' in self.commit_user else False

    def commit_user_fail(self):
        return True if '1' in self.commit_user else False

    def deploy_user_success(self):
        return True if '0' in self.deploy_user else False

    def deploy_user_fail(self):
        return True if '1' in self.deploy_user else False

    class Meta:
        ordering = ('name',)
        app_label = 'ftp_deploy'
        db_table = 'ftp_deploy_notification'



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Submit


class LoginForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={'placeholder': 'Username', 'required': 'required'}))
    password = forms.CharField(
        widget=forms.PasswordInput(
            render_value=False,
            attrs={'placeholder': 'Password', 'required': 'required'}))

    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = 'form-signin'
        self.helper.form_tag = False

        self.helper.layout = Layout(
            'username',
            'password',
            Submit('login', 'Log in', css_class='btn-lg btn-block'),
        )



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Service'
        db.create_table(u'ftp_deploy_service', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ftp_host', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('ftp_username', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('ftp_password', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('ftp_path', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('repo_source', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('repo_name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('repo_slug_name', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('repo_branch', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('repo_hook', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('secret_key', self.gf('django.db.models.fields.CharField')(default='ZTPBHR7OK0KglXN1w3pIQV2oSSSED3', unique=True, max_length=30)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('status_message', self.gf('django.db.models.fields.TextField')()),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'ftp_deploy', ['Service'])

        # Adding model 'Log'
        db.create_table(u'ftp_deploy_log', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('service', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ftp_deploy.Service'], blank=True)),
            ('payload', self.gf('django.db.models.fields.TextField')()),
            ('user', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('status_message', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('skip', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'ftp_deploy', ['Log'])


    def backwards(self, orm):
        # Deleting model 'Service'
        db.delete_table(u'ftp_deploy_service')

        # Deleting model 'Log'
        db.delete_table(u'ftp_deploy_log')


    models = {
        u'ftp_deploy.log': {
            'Meta': {'ordering': "('-created',)", 'object_name': 'Log'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'payload': ('django.db.models.fields.TextField', [], {}),
            'service': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['ftp_deploy.Service']", 'blank': 'True'}),
            'skip': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'status_message': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'user': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'ftp_deploy.service': {
            'Meta': {'object_name': 'Service'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'ftp_host': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ftp_password': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'ftp_path': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ftp_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'repo_branch': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'repo_hook': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'repo_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'repo_slug_name': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'repo_source': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'secret_key': ('django.db.models.fields.CharField', [], {'default': "'ohPqA5RjE8cxO7dtL5yWe04qZjfXEA'", 'unique': 'True', 'max_length': '30'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'status_message': ('django.db.models.fields.TextField', [], {}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['ftp_deploy']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Task'
        db.create_table('ftp_deploy_task', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=50)),
            ('service', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ftp_deploy.Service'])),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('ftp_deploy', ['Task'])

        # Deleting field 'Service.lock'
        db.delete_column(u'ftp_deploy_service', 'lock')


    def backwards(self, orm):
        # Deleting model 'Task'
        db.delete_table('ftp_deploy_task')

        # Adding field 'Service.lock'
        db.add_column(u'ftp_deploy_service', 'lock',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    models = {
        'ftp_deploy.log': {
            'Meta': {'ordering': "('-created',)", 'object_name': 'Log'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'payload': ('django.db.models.fields.TextField', [], {}),
            'service': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ftp_deploy.Service']", 'blank': 'True'}),
            'skip': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'status_message': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'user': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'ftp_deploy.notification': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Notification'},
            'commit_user': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'deploy_user': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'fail': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'success': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'ftp_deploy.service': {
            'Meta': {'object_name': 'Service'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'ftp_host': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ftp_password': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'ftp_path': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ftp_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notification': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ftp_deploy.Notification']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'repo_branch': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'repo_hook': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'repo_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'repo_slug_name': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'repo_source': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'secret_key': ('django.db.models.fields.CharField', [], {'default': "'Hm6NGAnG1ARvkejunTA0agR0slvgOt'", 'unique': 'True', 'max_length': '30'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_message': ('django.db.models.fields.TextField', [], {}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'ftp_deploy.task': {
            'Meta': {'object_name': 'Task'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'service': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ftp_deploy.Service']"})
        }
    }

    complete_apps = ['ftp_deploy']


from __future__ import absolute_import

import os

from celery import Celery
from django.conf import settings

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')

app = Celery('ftp_deploy')
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(settings.INSTALLED_APPS, related_name='tasks')



import os
import json
from ftplib import FTP
import tempfile
import time
from celery import task, current_task

from ftp_deploy.conf import *
from ftp_deploy.models import Log, Task
from .core import LockError
from .repo import commits_parser, repository_parser
from .ftp import ftp_connection
from .email import notification_success, notification_fail
from .curl import curl_connection


class Deploy(object):

    """Deploy method responsible for perform deploying"""

    def __init__(self, host, payload, service, task_name):
        self.host = host
        self.service = service
        self.task = Task.objects.get(name=task_name)
        self.data = json.loads(payload)
        self.json_string = payload
        self.files_count = commits_parser(
            self.data['commits'], self.service.repo_source).files_count()

        self.ftp_host = self.service.ftp_host
        self.ftp_username = self.service.ftp_username
        self.ftp_password = self.service.ftp_password
        self.ftp_path = self.service.ftp_path

        self.repo = repository_parser(self.data, self.service)
        self.repo_username, self.repo_password = self.repo.credentials()
        self.user = self.repo.deploy_name()

    def perform(self):
        """
            Perform ftp connection and choose repository perform method
            (bitbucket or github)
        """

        if self.user == 'Restore':
            self.service.get_logs_tree().delete()

        self.log = Log()
        self.log.payload = self.json_string
        self.log.service = self.service
        self.log.save()

        try:
            self.ftp = ftp_connection(
                self.ftp_host, self.ftp_username, self.ftp_password,
                self.ftp_path)

            if self.service.lock():
                raise LockError()

            self.task.active = True
            self.task.save()

            self.ftp.connect()

        except LockError, e:
            self.set_fail('Service Locked', e)
        except Exception, e:
            self.set_fail('FTP Connection', e)
        else:
            try:
                if self.service.repo_source == 'bb':
                    self.perform_bitbucket()

                if self.service.repo_source == 'gh':
                    self.perform_github()

            except Exception, e:
                self.set_fail(self.user, e)
            else:
                self.log.user = self.user
                self.log.status = True
                self.log.save()
                notification_success(self.host, self.service, self.json_string)

        finally:
            self.ftp.quit()
            self.service.check()
            self.service.save()
            self.task.delete()

    def perform_bitbucket(self):
        """perform bitbucket deploy"""
        curl = curl_connection(self.repo_username, self.repo_password)
        curl.authenticate()
        i = 0
        for commit in self.data['commits']:
            for files in commit['files']:
                file_path = files['file']

                self.update_progress(i, file_path)
                i += 1

                if files['type'] == 'removed':
                    self.ftp.remove_file(file_path)
                else:
                    url = 'https://api.bitbucket.org/1.0/repositories%sraw/%s/%s' % (
                        self.data['repository']['absolute_url'],
                        commit['node'], file_path)
                    url = str(url.encode('utf-8'))

                    value = curl.perform(url)
                    self.create_file(file_path, value)

        curl.close()

    def perform_github(self):
        curl = curl_connection(self.repo_username, self.repo_password)
        curl.authenticate()
        i = 0

        for commit in self.data['commits']:

            for file in commit['removed']:
                self.update_progress(i, file)
                i += 1

                self.ftp.remove_file(file)

            for file in commit['added'] + commit['modified']:
                self.update_progress(i, file)
                i += 1

                url = 'https://raw.github.com/%s/%s/%s/%s' % (
                    self.data['repository']['owner']['name'],
                    self.data['repository']['name'], commit['id'], file)

                url = str(url.encode('utf-8'))
                value = curl.perform(url)
                self.create_file(file, value)

        curl.close()

    def update_progress(self, i, file_path):
        progress_percent = int(100 * float(i) / float(self.files_count))
        current_task.update_state(state='PROGRESS', meta={
                                  'status': progress_percent,
                                  'file': os.path.basename(file_path)})

    def create_file(self, file_path, value):
        temp_file = tempfile.NamedTemporaryFile(delete=False)
        temp_file.write(value)
        temp_file.close()
        temp_file = open(temp_file.name, 'rb')

        self.ftp.make_dirs(file_path)
        self.ftp.create_file(file_path, temp_file)

        temp_file.close()
        os.unlink(temp_file.name)

    def set_fail(self, user, message):
        self.log.user = user
        self.log.status_message = message
        self.log.status = False
        self.log.save()
        notification_fail(self.host, self.service, self.json_string, message)



import json
from django.db import models

from ftp_deploy.utils.repo import commits_parser
from .service import Service


class Log(models.Model):
    service = models.ForeignKey(Service, blank=True)
    payload = models.TextField()
    user = models.CharField(max_length=200)
    status = models.BooleanField(default=False)
    status_message = models.TextField(blank=True)
    created = models.DateTimeField(auto_now_add=True)
    skip = models.BooleanField(default=False)

    def commits_info(self):
        commits = json.loads(self.payload)['commits']
        return commits_parser(commits, self.service.repo_source).commits_info()

    class Meta:
        ordering = ('-created',)
        app_label = 'ftp_deploy'
        db_table = 'ftp_deploy_log'



from __future__ import absolute_import

from .celery import app
from .utils.deploy import Deploy


@app.task
def deploy_task(host, payload, service):
    deploy = Deploy(host, payload, service, deploy_task.request.id)
    deploy.perform()







from django.conf.urls import patterns, url
from .views import DashboardView, LogView, LogSkipDeployView, RepoAPIView
from .views import (ServiceManageView, ServiceRestoreView, ServiceEditView,
                    ServiceAddView, ServiceStatusView, ServiceDeleteView,
                    ServiceNotificationView)
from .views import (NotificationView, NotificationAddView,
                    NotificationEditView, NotificationDeleteView)
from .views import loginView, logoutView


urlpatterns = patterns(
    '',
    url(r'^$', loginView.as_view(), name='ftpdeploy_login'),
    url(r'^logout/$', logoutView.as_view(), name='ftpdeploy_logout'),

    url(r'^dashboard/$', DashboardView.as_view(), name='ftpdeploy_dashboard'),

    url(r'^service/add$', ServiceAddView.as_view(),
        name='ftpdeploy_service_add'),
    url(r'^service/(?P<pk>\d+)/manage$', ServiceManageView.as_view(),
        name='ftpdeploy_service_manage'),
    url(r'^service/(?P<pk>\d+)/edit$', ServiceEditView.as_view(),
        name='ftpdeploy_service_edit'),
    url(r'^service/(?P<pk>\d+)/delete$', ServiceDeleteView.as_view(),
        name='ftpdeploy_service_delete'),
    url(r'^service/(?P<pk>\d+)/status/$', ServiceStatusView.as_view(),
        name='ftpdeploy_service_status'),
    url(r'^service/(?P<pk>\d+)/restore/$', ServiceRestoreView.as_view(),
        name='ftpdeploy_service_restore'),
    url(r'^service/(?P<pk>\d+)/notification/$', ServiceNotificationView.as_view(),
        name='ftpdeploy_service_notification'),
    url(r'^service/(?P<pk>\d+)/repoapi/(?P<repo>.{2})/$', RepoAPIView.as_view(),
        name='ftpdeploy_repo_api'),

    url(r'^log/$', LogView.as_view(), name='ftpdeploy_log'),
    url(r'^log/(?P<pk>\d+)/skip/$', LogSkipDeployView.as_view(),
        name='ftpdeploy_log_skip'),

    url(r'^notification/$', NotificationView.as_view(),
        name='ftpdeploy_notification'),
    url(r'^notification/add$', NotificationAddView.as_view(),
        name='ftpdeploy_notification_add'),
    url(r'^notification/(?P<pk>\d+)/edit$', NotificationEditView.as_view(),
        name='ftpdeploy_notification_edit'),
    url(r'^notification/(?P<pk>\d+)/delete$', NotificationDeleteView.as_view(),
        name='ftpdeploy_notification_delete'),
)



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Field

from ftp_deploy.models import Notification


class NotificationForm(forms.ModelForm):

    success = forms.CharField(
        widget=forms.TextInput(
            attrs={'size': '150'}), help_text='Comma separated list of emails',
        required=False)
    fail = forms.CharField(
        widget=forms.TextInput(
            attrs={'size': '150'}), help_text='Comma separated list of emails',
        required=False)
    deploy_user = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple(),
        choices=((0, 'Success'), (1, 'Fail')), required=False)
    commit_user = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple(),
        choices=((0, 'Success'), (1, 'Fail')), required=False)

    def __init__(self, *args, **kwargs):
        super(NotificationForm, self).__init__(*args, **kwargs)

        self.helper_header = FormHelper()
        self.helper_header.html5_required = True
        self.helper_header.form_tag = False

        self.helper_header.layout = Layout(
            'name',
            Field('success', type="hidden"),
            Field('fail', type="hidden")
        )

        self.helper_user = FormHelper()
        self.helper_user.form_tag = False

        self.helper_user.layout = Layout(
            Field('deploy_user',
                  template='ftp_deploy/notification/form-user-field.html'),
            Field('commit_user',
                  template='ftp_deploy/notification/form-user-field.html')
        )

    class Meta:
        model = Notification



import pycurl
import certifi
import StringIO


class curl_connection(object):

    """Helper for curl connections"""

    def __init__(self, username, password):
        self.username = username
        self.password = password

    def authenticate(self):
        """Authenticate curl connection"""
        self.curl = pycurl.Curl()
        self.curl.setopt(pycurl.CAINFO, certifi.where())
        self.curl.setopt(self.curl.USERPWD, '%s:%s' % (self.username,
                         self.password))

    def perform(self, url):
        """Perform get request and return respond value"""
        b = StringIO.StringIO()
        self.curl.setopt(self.curl.URL, str(url))
        self.curl.setopt(self.curl.WRITEFUNCTION, b.write)
        self.curl.perform()
        return b.getvalue()

    def perform_post(self, url, post):
        """Perform post request"""
        self.curl.setopt(self.curl.URL, str(url))
        self.curl.setopt(self.curl.POSTFIELDS, post)
        self.curl.perform()

    def get_http_code(self):
        """Return curl HTTP Code"""
        return self.curl.getinfo(pycurl.HTTP_CODE)

    def close(self):
        """Close curl connection"""
        self.curl.close()



import json

from django.views.generic.base import View
from django.views.generic.detail import SingleObjectMixin
from django.views.generic import (ListView, UpdateView, DeleteView,
                                  DetailView, CreateView)
from django.core.urlresolvers import reverse_lazy, reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib import messages
from django.db.models import Max
from django.http import HttpResponse, HttpResponseRedirect, Http404

from braces.views import JSONResponseMixin, LoginRequiredMixin

from ftp_deploy.conf import *
from ftp_deploy.models import Service
from ftp_deploy.utils.repo import commits_parser
from ftp_deploy.server.forms import ServiceForm, ServiceNotificationForm


class DashboardView(LoginRequiredMixin, ListView):

    """View for dashboard"""

    model = Service
    queryset = Service.objects.all().select_related().order_by(
        "status", "-log__created").annotate(date=Max('log__created'))
    context_object_name = 'services'
    template_name = "ftp_deploy/dashboard.html"
    paginate_by = 25

    def post(self, request, *args, **kwargs):
        services = self.get_queryset()
        if self.request.POST['services']:
            services = services.filter(pk=self.request.POST['services'])
        return render_to_response('ftp_deploy/service/list.html', locals(),
                                  context_instance=RequestContext(request))


class ServiceManageView(LoginRequiredMixin, DetailView):

    """View for manage services"""

    model = Service
    context_object_name = 'service'
    template_name = "ftp_deploy/service/manage.html"

    def get_context_data(self, **kwargs):
        context = super(ServiceManageView, self).get_context_data(**kwargs)
        context['recent_logs'] = self.object.log_set.all()[:15]
        context['fail_logs'] = self.object.log_set.filter(
            status=0).filter(skip=0)
        return context


class ServiceAddView(LoginRequiredMixin, CreateView):

    """View for add serives"""
    model = Service
    form_class = ServiceForm
    template_name = "ftp_deploy/service/form.html"

    def form_valid(self, form):
        messages.add_message(self.request, messages.SUCCESS,
                             'Service has been added.')
        return super(ServiceAddView, self).form_valid(form)

    def get_success_url(self):
        self.object.check()
        self.object.save()
        return reverse('ftpdeploy_service_manage',
                       kwargs={'pk': self.object.pk})


class ServiceEditView(LoginRequiredMixin, UpdateView):

    """View for edit services"""
    model = Service
    form_class = ServiceForm
    template_name = "ftp_deploy/service/form.html"

    def form_valid(self, form):
        self.object.check()
        self.object.save()
        messages.add_message(self.request, messages.SUCCESS,
                             'Service has been updated.')
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('ftpdeploy_service_manage',
                       kwargs={'pk': self.kwargs['pk']})


class ServiceDeleteView(LoginRequiredMixin, DeleteView):

    """View for delete services"""

    model = Service
    success_url = reverse_lazy('ftpdeploy_dashboard')
    template_name = "ftp_deploy/service/delete.html"

    def delete(self, request, *args, **kwargs):
        messages.add_message(request, messages.SUCCESS,
                             'Service has been removed.')
        return super(ServiceDeleteView, self).delete(request, *args, **kwargs)


class ServiceStatusView(JSONResponseMixin, LoginRequiredMixin,
                        SingleObjectMixin, View):

    """View for update(save) and check service status"""

    model = Service

    def post(self, request, *args, **kwargs):
        service = self.get_object()
        service.check()
        service.save()
        response = request.POST.get('response', '')

        if response == 'list':
            services = [service]
            return render_to_response('ftp_deploy/service/list.html', locals(),
                                      context_instance=RequestContext(request))

        if response == 'manage':
            manage_view = ServiceManageView()
            manage_view.object = service
            context = manage_view.get_context_data()
            return render_to_response(
                'ftp_deploy/service/manage.html',
                context, context_instance=RequestContext(request))

        if response == 'json':
            context = {
                'status': service.status,
                'status_message': service.status_message,
                'updated': service.updated
            }
            return self.render_json_response(context)

        raise Http404


class ServiceRestoreView(LoginRequiredMixin, DetailView):

    """"View for build restore path for service"""

    model = Service
    prefetch_related = ["log_set"]
    template_name = "ftp_deploy/service/restore-modal.html"

    def get_context_data(self, **kwargs):
        context = super(ServiceRestoreView, self).get_context_data(**kwargs)

        service = self.get_object()
        logs = service.get_logs_tree()

        # init payload dictionary
        context['payload'] = json.loads(logs[0].payload)

        if service.repo_source == 'bb':
            context['payload']['user'] = 'Restore'
        elif service.repo_source == 'gh':
            context['payload']['pusher']['name'] = 'Restore'

        context['service'] = service

        commits = list()
        for log in logs:
            payload = json.loads(log.payload)
            commits += payload['commits']

        context['payload']['commits'] = commits
        context['payload'] = json.dumps(context['payload'])

        context['files_added'], context['files_modified'], context['files_removed'] = commits_parser(commits, service.repo_source).file_diff()
        context['commits_info'] = commits_parser(commits, service.repo_source).commits_info()

        return context

    def post(self, request, *args, **kwargs):
        if self.get_object().lock():
            return HttpResponse(status=500)

        return HttpResponse(
            reverse('ftpdeploy_deploy',
                    kwargs={'secret_key': self.get_object().secret_key}))


class ServiceNotificationView(LoginRequiredMixin, UpdateView):

    model = Service
    form_class = ServiceNotificationForm
    template_name = "ftp_deploy/notification/notification-modal.html"

    def form_valid(self, form):
        messages.add_message(self.request, messages.SUCCESS,
                             'Service notification has been updated.')
        return super(ServiceNotificationView, self).form_valid(form)

    def get_success_url(self):
        return reverse('ftpdeploy_service_manage',
                       kwargs={'pk': self.kwargs['pk']})



from django.views.generic import ListView
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.base import View
from django.shortcuts import render_to_response
from django.template import RequestContext
from braces.views import JSONResponseMixin, LoginRequiredMixin

from ftp_deploy.models import Log, Service


class LogView(LoginRequiredMixin, ListView):

    """View for display logs"""

    model = Log
    context_object_name = 'logs'
    template_name = "ftp_deploy/log/log.html"
    paginate_by = 25

    def get_context_data(self, **kwargs):
        context = super(LogView, self).get_context_data(**kwargs)
        context['service_list'] = Service.objects.all().values('repo_name',
                                                               'pk')
        return context

    def post(self, request, *args, **kwargs):
        logs = self.get_queryset()
        if self.request.POST['services']:
            logs = logs.filter(service__pk=self.request.POST['services'])

        if not int(self.request.POST['status']):
            logs = logs.filter(status=self.request.POST['status'])
        return render_to_response('ftp_deploy/log/list.html', locals(),
                                  context_instance=RequestContext(request))


class LogSkipDeployView(LoginRequiredMixin, JSONResponseMixin,
                        SingleObjectMixin, View):

    """View for skip fail logs"""

    model = Log

    def post(self, request, *args, **kwargs):
        log = self.get_object()
        log.skip = True
        log.save()
        return self.render_json_response({'status': 'success'})



from django.contrib import admin

from .models import Log, Service, Notification
from ftp_deploy.server.forms import NotificationForm, ServiceForm


class ServiceAdmin(admin.ModelAdmin):
    form = ServiceForm
    list_display = ('repo_name', 'repo_branch', 'hook_url',
                    'status_message_html', 'status')
    fieldsets = (
        ('FTP Settings', {
            'fields': ('ftp_host', ('ftp_username', 'ftp_password'),
                       'ftp_path')
        }),
        ('Repository', {
            'classes': ('',),
            'fields': ('repo_source', 'repo_name', 'repo_branch')
        }),
        ('Notification', {
            'classes': ('',),
            'fields': ('notification',)
        }),
        ('Security', {
            'classes': ('collapse',),
            'fields': ('secret_key',)
        }),
    )

    def status_message_html(self, obj):
        return obj.status_message
    status_message_html.allow_tags = True
    status_message_html.short_description = 'Status Message'


class LogAdmin(admin.ModelAdmin):
    list_display = ('id', 'created', 'user', 'status_message_html', 'status')
    readonly_fields = ('service', 'created', 'user', 'status',
                       'status_message_html')
    exclude = ('payload',)

    def has_add_permission(self, request):
        return False

    def status_message_html(self, obj):
        return obj.status_message
    status_message_html.allow_tags = True
    status_message_html.short_description = 'Status Message'


class NotificationAdmin(admin.ModelAdmin):
    form = NotificationForm
    list_display = ('name',)

admin.site.register(Notification, NotificationAdmin)

admin.site.register(Service, ServiceAdmin)
admin.site.register(Log, LogAdmin)




from django.db import models

from .service import Service


class Task(models.Model):
    name = models.CharField(max_length=50, unique=True)
    service = models.ForeignKey(Service)
    active = models.BooleanField(default=False)

    class Meta:
        app_label = 'ftp_deploy'
        db_table = 'ftp_deploy_task'




class check(object):

    """
    Wrap function in try/except

    Return True with error message (with prefix) if raise Exception,
    otherwise return False and empty string
    """

    def __init__(self, prefix):
        self.prefix = prefix

    def __call__(self, fn):

        def wrapped(*args):
            try:
                fn(*args)
            except Exception, e:
                return True, '<b>%s:</b> %s' % (self.prefix, e)
            else:
                return False, ''

        return wrapped




from django.views.generic.base import View
from django.views.generic.detail import SingleObjectMixin

from braces.views import JSONResponseMixin, LoginRequiredMixin

from ftp_deploy.conf import *
from ftp_deploy.models import Service
from ftp_deploy.utils.repo import repository_api


class RepoAPIView(LoginRequiredMixin, JSONResponseMixin, SingleObjectMixin,
                  View):

    """View for managing BitBucket API"""

    model = Service

    def dispatch(self, request, repo, *args, **kwargs):
        self.repo_api = repository_api(repo)
        return super(RepoAPIView, self).dispatch(request, repo, * args,
                                                 **kwargs)

    def post(self, request, repo, *args, **kwargs):
        if self.request.POST['data'] == 'respositories':
            context = self.repo_api.repositories()
        elif self.request.POST['data'] == 'addhook':
            context = self.repo_api.add_hook(self.get_object(), request)

        return self.render_json_response(context)



from django import forms
from django.core.urlresolvers import reverse

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Field, Submit, Div

from ftp_deploy.models import Service


class ServiceForm(forms.ModelForm):

    """Add/Edit service form"""

    def __init__(self, *args, **kwargs):
        super(ServiceForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_id = 'service-form'
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-sm-3'
        self.helper.field_class = 'col-sm-9'
        self.helper.html5_required = True

        self.helper.layout = Layout(
            Fieldset('FTP Settings',
                     'ftp_host',
                     'ftp_username',
                     'ftp_password',
                     'ftp_path'
                     ),
            Fieldset('Repository',
                     Field('repo_source',
                           data_action=reverse(
                               'ftpdeploy_repo_api', args=(0, '__'))),
                     'repo_name',
                     'repo_slug_name',
                     'repo_branch'
                     ),
            Fieldset('Notification',
                     'notification'
                     ),
            Fieldset('Security',
                     'secret_key'
                     ),
            Div(
                Div(
                    Submit('save', 'Submit', css_class='pull-right'),
                    css_class='col-sm-12'
                ),
                css_class='row'
            )

        )

    def clean_ftp_path(self):
        data = self.cleaned_data['ftp_path']
        if not data.endswith('/'):
            data = '%s/' % data
        return data

    class Meta:
        model = Service
        exclude = ['status', 'status_date', 'status_message']
        widgets = {
            'ftp_password': forms.PasswordInput(render_value=True),
        }


class ServiceNotificationForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(ServiceNotificationForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_id = 'notification-form'
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'hide'
        self.helper.field_class = 'col-sm-12'
        self.helper.form_tag = False

        self.helper.layout = Layout(
            'notification'
        )

    class Meta:
        model = Service
        fields = ['notification']



import pycurl
import json
import re

import core
from .curl import curl_connection
from .decorators import check
from ftp_deploy.conf import *


class repository_parser(object):

    def __init__(self, data, service):
        self.service = service
        self.data = data
        self.service_repo = service.repo_source

    def credentials(self):
        """return login details"""
        if self.service_repo == 'bb':
            return (BITBUCKET_SETTINGS['username'], BITBUCKET_SETTINGS['password'])
        elif self.service_repo == 'gh':
            return GITHUB_SETTINGS['username'], GITHUB_SETTINGS['password']

    def check_branch(self):
        """check if payload branch match set branch"""
        if self.service_repo == 'bb':
            last_commit = len(self.data['commits']) - 1
            if (self.data['commits'][last_commit]['branch'] ==
                    self.service.repo_branch):
                return True
        elif self.service_repo == 'gh':
            ref = self.data['ref'].split('/')
            if ref[2] == self.service.repo_branch:
                return True

        return False

    def deploy_email(self):
        """return email of deploy user"""

        if self.service_repo == 'bb':
            if self.data['user'] == 'Restore':
                return []
            try:
                curl = curl_connection(BITBUCKET_SETTINGS['username'],
                                       BITBUCKET_SETTINGS['password'])
                curl.authenticate()
                url = 'https://bitbucket.org/api/1.0/users/%s/emails' % self.data['user']
                context = json.loads(curl.perform(url))
                return [context[0]['email']]
            except Exception, e:
                return []
        elif self.service_repo == 'gh':
            if self.data['pusher']['name'] == 'Restore':
                return []
            return self.data['pusher']['email']

    def deploy_name(self):
        """ return username of deploy user"""
        if self.service_repo == 'bb':
            return self.data['user']
        elif self.service_repo == 'gh':
            return self.data['pusher']['name']


class repository_api(object):

    def __init__(self, service_repo):
        self.service_repo = service_repo
        self.curl_init()

    def curl_init(self):
        if self.service_repo == 'bb':
            self.username = BITBUCKET_SETTINGS['username']
            self.password = BITBUCKET_SETTINGS['password']
        elif self.service_repo == 'gh':
            self.username = GITHUB_SETTINGS['username']
            self.password = GITHUB_SETTINGS['password']

        self.curl = curl_connection(self.username, self.password)
        self.curl.authenticate()

    def repositories(self):
        """Load list of repositories from repository account"""

        if(self.service_repo == 'bb'):
            url = 'https://bitbucket.org/api/1.0/user/repositories'
        elif(self.service_repo == 'gh'):
            url = 'https://api.github.com/user/repos'

        context = self.curl.perform(url)
        return context

    def add_hook(self, service, request):
        """Add hook and change repo_hook flag for service"""

        if(self.service_repo == 'bb'):
            url = 'https://api.bitbucket.org/1.0/repositories/%s/%s/services/' % (
                self.username, service.repo_slug_name)
            post = 'type=POST&URL=%s%s' % (core.absolute_url(request).build(),
                                           service.hook_url())
        elif(self.service_repo == 'gh'):
            url = 'https://api.github.com/repos/%s/%s/hooks' % (
                self.username, service.repo_slug_name)

            data = {
                "name": 'web',
                "active": True,
                "config": {
                    "url": '%s%s' % (core.absolute_url(request).build(),
                                     service.hook_url()),
                    "content_type": "json"
                }
            }

            post = json.dumps(data)

        service.repo_hook = True
        service.save()
        context = self.curl.perform_post(url, post)
        return context


class commits_parser(object):

    """Commit parser for list of commits.
        Take commits dictionary captured from payload"""

    def __init__(self, commits, service_repo):
        self.commits = commits
        self.service_repo = service_repo

    def commits_info(self):
        """Return commits details list in format [message,author,raw_node]"""
        output = list()
        if self.service_repo == 'bb':
            [output.append([commit['message'], commit['author'],
                            commit['raw_node']]) for commit in reversed(self.commits)]
        elif self.service_repo == 'gh':
            [output.append([commit['message'], commit['author']['name'],
                           commit['id']]) for commit in reversed(self.commits)]

        return output

    def email_list(self):
        """Return email list from raw_author, limited to unique emails"""
        output = list()

        if self.service_repo == 'bb':
            for commit in self.commits:
                email = re.search('%s(.*)%s' % ('<', '>'),
                                  commit['raw_author']).group(1)
                output.append(email) if email not in output else False
        elif self.service_repo == 'gh':
            for commit in self.commits:
                email = commit['author']['email']
                output.append(email) if email not in output else False

        return output

    def file_diff(self):
        """Return files list grouped by added, modified and removed.
            Respect order of commits"""
        added, removed, modified = list(), list(), list()

        if self.service_repo == 'bb':
            for commit in self.commits:
                for file in commit['files']:
                    if file['type'] == 'added':
                        added.append(file['file']) if file['file'] not in added else False
                        removed.remove(file['file']) if file['file'] in removed else False
                    elif file['type'] == 'modified':
                        modified.append(file['file']) if file['file'] not in modified and file['file'] not in added else False
                    elif file['type'] == 'removed':
                        removed.append(file['file']) if file['file'] not in removed + added else False
                        added.remove(file['file']) if file['file'] in added else False
                        modified.remove(file['file']) if file['file'] in modified else False

        elif self.service_repo == 'gh':
            for commit in self.commits:
                for file in commit['added']:
                    added.append(file) if file not in added else False
                    removed.remove(file) if file in removed else False
                for file in commit['modified']:
                    modified.append(file) if file not in modified and file not in added else False
                for file in commit['removed']:
                    removed.append(file) if file not in removed + added else False
                    added.remove(file) if file in added else False
                    modified.remove(file) if file in modified else False

        return added, modified, removed

    def files_count(self):
        count = 0

        if self.service_repo == 'bb':
            for commit in self.commits:
                count += len(commit['files'])
        elif self.service_repo == 'gh':
            for commit in self.commits:
                count += len(commit['added']) + len(commit['modified']) + len(commit['removed'])

        return count


class bitbucket_check(curl_connection):

    """Bitbucket check class contain all checking points for bitbucket,
        return True if fail"""

    def __init__(self, username, password, service):
        super(bitbucket_check, self).__init__(username, password)
        self.service = service

    @check('Bitbucket')
    def check_authentication(self):
        self.authenticate()
        self.perform('https://bitbucket.org/api/1.0/user/repositories')
        if self.curl.getinfo(pycurl.HTTP_CODE) != 200:
            raise Exception("Login Fail")

    @check('Bitbucket')
    def check_repo_exist(self):
        self.authenticate()
        repos = json.loads(self.perform('https://bitbucket.org/api/1.0/user/repositories'))
        for repo in repos:
            if repo['slug'] == self.service.repo_slug_name:
                return False, ''
        raise Exception("Repository %s doesn't exist" %
                        self.service.repo_slug_name)

    @check('Bitbucket')
    def check_branch_exist(self):
        self.authenticate()
        url = 'https://bitbucket.org/api/1.0/repositories/%s/%s/branches' % (
            self.username, self.service.repo_slug_name)
        branches = json.loads(self.perform(url))
        try:
            branches[self.service.repo_branch]
        except KeyError, e:
            raise Exception("Branch %s doesn't exist" %
                            self.service.repo_branch)

    @check('Bitbucket')
    def check_hook_exist(self):
        self.authenticate()
        url = 'https://bitbucket.org/api/1.0/repositories/%s/%s/services' % (
            self.username, self.service.repo_slug_name)
        hooks = json.loads(self.perform(url))

        if type(hooks) == list:
            for hook in hooks:
                if len(hook['service']['fields']) > 0:
                    value = hook['service']['fields'][0]['value']
                    if (value.find(str(self.service.hook_url())) != -1
                            and hook['service']['type'] == 'POST'):
                        return False, ''
        raise Exception("Hook is not set up")

    def check_all(self):
        status = self.check_authentication()
        if status[0] is True:
            return status

        status = self.check_repo_exist()
        if status[0] is True:
            return status

        status = self.check_branch_exist()
        if status[0] is True:
            return status

        return False, ''


class github_check(curl_connection):

    """Bitbucket check class contain all checking points for bitbucket,
        return True if fail"""

    def __init__(self, username, password, service):
        super(github_check, self).__init__(username, password)
        self.service = service

    @check('Github')
    def check_authentication(self):
        self.authenticate()
        self.perform('https://api.github.com/user/repos')
        if self.curl.getinfo(pycurl.HTTP_CODE) != 200:
            raise Exception("Login Fail")

    @check('Github')
    def check_repo_exist(self):
        self.authenticate()
        repos = json.loads(self.perform('https://api.github.com/user/repos'))
        for repo in repos:
            if repo['name'] == self.service.repo_slug_name:
                return False, ''
        raise Exception("Repository %s doesn't exist" %
                        self.service.repo_slug_name)

    @check('Github')
    def check_branch_exist(self):
        self.authenticate()
        url = 'https://api.github.com/repos/%s/%s/branches' % (
            self.username, self.service.repo_slug_name)
        branches = json.loads(self.perform(url))
        for branch in branches:
            if branch['name'] == self.service.repo_branch:
                return False, ''
        raise Exception("Branch %s doesn't exist" % self.service.repo_branch)

    @check('Github')
    def check_hook_exist(self):
        self.authenticate()
        url = 'https://api.github.com/repos/%s/%s/hooks' % (
            self.username, self.service.repo_slug_name)
        hooks = json.loads(self.perform(url))

        if type(hooks) == list:
            for hook in hooks:
                value = hook['config']['url']
                if (value.find(str(self.service.hook_url())) != -1
                        and hook['name'] == "web"):
                    return False, ''

        raise Exception("Hook is not set up")

    def check_all(self):
        status = self.check_authentication()
        if status[0] is True:
            return status

        status = self.check_repo_exist()
        if status[0] is True:
            return status

        status = self.check_branch_exist()
        if status[0] is True:
            return status

        return False, ''



from distutils.core import setup

setup(
    name='django-ftp-deploy',
    version='1.4.1',
    author='Lukasz Pakula',
    author_email='lukaszpakula.pl@gmail.com',
    packages=[
        'ftp_deploy',
        'ftp_deploy.migrations',
        'ftp_deploy.models',
        'ftp_deploy.utils',
        'ftp_deploy.server',
        'ftp_deploy.server.forms',
        'ftp_deploy.server.templatetags',
        'ftp_deploy.server.views'
    ],
    include_package_data=True,
    url='http://bitbucket.org/lpakula/django-ftp-deploy',
    license='LICENSE.txt',
    description='Auto FTP deployment for django.',
    long_description=open('README.rst').read(),
    install_requires=[
        "pycurl == 7.19.0.1",
        "certifi == 0.0.8",
        "django-braces == 1.2.1",
        "django-crispy-forms==1.4.0",
        "celery==3.1.8"
    ],
)



import re

from django import template
from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch

register = template.Library()

@register.simple_tag(takes_context=True)
def active(context, pattern_or_urlname):
    """Tag  provide active class for menu elements"""
    try:
        pattern = '^' + reverse(pattern_or_urlname)
    except NoReverseMatch:
        pattern = pattern_or_urlname
    path = context['request'].path
    if re.search(pattern, path):
        return 'active'
    return ''


from django.conf import settings

BITBUCKET_SETTINGS = getattr(settings, 'DEPLOY_BITBUCKET_SETTINGS', {
    'username' 		: '',
    'password' 		: '',
})

GITHUB_SETTINGS = getattr(settings, 'DEPLOY_GITHUB_SETTINGS', {
    'username' 		: '',
    'password' 		: '',
})



import json

from abc import ABCMeta, abstractmethod
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string

from ftp_deploy.conf import *
from .repo import commits_parser, repository_parser


class notification():

    """Notification abstract class, take three arguments, host,
        service object and payload json string"""
    __metaclass__ = ABCMeta

    def __init__(self, host, service, payload):
        self.host = host
        self.service = service
        self.payload = json.loads(payload)
        self.commits = self.payload['commits']
        self.from_email = 'noreply@ftpdeploy.com'
        self.repo = repository_parser(self.payload, self.service)
        self.send()

    @property
    def template_html(self):
        raise NotImplementedError

    @property
    def template_text(self):
        raise NotImplementedError

    @abstractmethod
    def subject(self):
        pass

    @abstractmethod
    def emails(self):
        pass

    @abstractmethod
    def context(self):
        pass

    def send(self):
        """Sent method process emails from list returned by emails() method"""

        for recipient in self.emails():
            text_content = render_to_string(self.template_text, self.context())
            html_content = render_to_string(self.template_html, self.context())

            msg = EmailMultiAlternatives(self.subject(), text_content,
                                         self.from_email, [recipient])
            msg.attach_alternative(html_content, "text/html")
            msg.send()


class notification_success(notification):

    """Notification class for success"""

    template_html = 'ftp_deploy/email/email_success.html'
    template_text = 'ftp_deploy/email/email_success.txt'

    def subject(self):
        return '%s - Deploy Successfully' % self.service

    def emails(self):
        emails_list = list()
        notifications = self.service.notification

        if notifications:
            emails_list += notifications.get_success()

            if notifications.deploy_user_success():
                emails_list += self.repo.deploy_email()

            if notifications.commit_user_success():
                emails_list += commits_parser(
                    self.commits, self.service.repo_source).email_list()

        return list(set(emails_list))

    def context(self):
        context = dict()
        context['service'] = self.service
        context['host'] = self.host
        context['commits_info'] = commits_parser(
            self.commits, self.service.repo_source).commits_info()
        context['files_added'], context['files_modified'], context[
            'files_removed'] = commits_parser(
            self.commits, self.service.repo_source).file_diff()

        return context


class notification_fail(notification):

    """Notification class for fail"""

    template_html = 'ftp_deploy/email/email_fail.html'
    template_text = 'ftp_deploy/email/email_fail.txt'

    def __init__(self, host, service, payload, error):
        self.error = error
        super(notification_fail, self).__init__(host, service, payload)

    def subject(self):
        return '%s - Deploy Fail' % self.service

    def emails(self):
        emails_list = list()
        notifications = self.service.notification

        if notifications:

            emails_list += notifications.get_fail()

            if notifications.deploy_user_fail():
                emails_list += self.repo.deploy_email()

            if notifications.commit_user_fail():
                emails_list += commits_parser(
                    self.commits, self.service.repo_source).email_list()

        return list(set(emails_list))

    def context(self):
        context = dict()
        context['host'] = self.host
        context['service'] = self.service
        context['error'] = self.error
        return context



import random
import string

from django.db import models
from django.core.urlresolvers import reverse

from ftp_deploy.utils.core import service_check
from ftp_deploy.conf import *
from .notification import Notification


def repo_choices():
    choices = tuple()
    if BITBUCKET_SETTINGS['username'] and BITBUCKET_SETTINGS['password']:
        choices = choices + (('bb', 'BitBucket'),)

    if GITHUB_SETTINGS['username'] and GITHUB_SETTINGS['password']:
        choices = choices + (('gh', 'Github'),)

    return choices


class Service(models.Model):

    ftp_host = models.CharField('Host', max_length=255)
    ftp_username = models.CharField('Username', max_length=50)
    ftp_password = models.CharField('Password', max_length=50)
    ftp_path = models.CharField('Path', max_length=255)

    repo_source = models.CharField('Source', max_length=10,
                                   choices=repo_choices())
    repo_name = models.CharField('Respository Name', max_length=50)
    repo_slug_name = models.SlugField('Respository Slug', max_length=50)
    repo_branch = models.CharField('Branch', max_length=50)
    repo_hook = models.BooleanField(default=False)

    secret_key = models.CharField('Secret Key', unique=True, max_length=30,
                                  default=lambda: ''.join(random.choice(
                                      string.letters + string.digits) for x in range(30)))

    status = models.BooleanField(default=True)
    status_message = models.TextField()
    notification = models.ForeignKey(Notification, null=True, blank=True,
                                     on_delete=models.SET_NULL)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.repo_name

    def deploys(self):
        return self.log_set.filter(status=True).count()

    def fail_deploys(self):
        return self.log_set.filter(status=False).filter(skip=False).count()

    def skipped_deploys(self):
        return self.log_set.filter(status=False).filter(skip=True).count()

    def latest_log_date(self):
        return self.log_set.latest('created').created

    def latest_log_user(self):
        return self.log_set.latest('created').user

    def hook_url(self):
        return reverse('ftpdeploy_deploy', kwargs={'secret_key':
                       self.secret_key})

    def get_logs_tree(self):
        """get logs tree for restore deploys.
            Include all logs since first fail apart of skiped."""
        first_fail_log = self.log_set.filter(status=0).order_by('pk')[:1]
        logs = self.log_set.filter(skip=0).filter(
            pk__gte=first_fail_log[0].pk).order_by('pk')
        return logs

    def lock(self):
        return self.task_set.filter(active=True).exists()

    def has_queue(self):
        return self.task_set.all().exists()

    def check(self, **kwargs):

        message = list()
        fails, message = service_check(self).check_all()

        if fails[2]:
            self.repo_hook = False
        else:
            self.repo_hook = True

        if True in fails:
            self.status_message = '<br>'.join(message)
            self.status = False
        else:
            self.status = True
            self.status_message = ''

    class Meta:
        app_label = 'ftp_deploy'
        db_table = 'ftp_deploy_service'



from django.http import HttpResponseRedirect
from django.contrib import messages
from django.contrib.auth import authenticate
from django.contrib.auth import login as login_user
from django.contrib.auth import logout
from django.core.urlresolvers import reverse, reverse_lazy
from django.shortcuts import redirect
from django.views.generic import FormView, View

from ftp_deploy.server.forms import LoginForm


class loginView(FormView):

    """View for login"""

    form_class = LoginForm
    template_name = 'ftp_deploy/login/login.html'
    success_url = reverse_lazy('ftpdeploy_dashboard')

    def get(self, request, *args, **kwargs):
        if self.request.user.is_authenticated():
            return HttpResponseRedirect(reverse('ftpdeploy_dashboard'))
        return super(loginView, self).get(request, *args, **kwargs)

    def form_valid(self, form):
        username = form.cleaned_data['username']
        password = form.cleaned_data['password']
        user = authenticate(username=username, password=password)
        if user is not None:
            login_user(self.request, user)
        else:
            messages.error(self.request, 'Username or Password incorrect.')
            return HttpResponseRedirect(reverse('ftpdeploy_login'))
        return super(loginView, self).form_valid(form)


class logoutView(View):

    """View for logout"""

    def get(self, request, *args, **kwargs):
        logout(request)
        return redirect(reverse('ftpdeploy_login'))
